#!/usr/local/bin/ruby -Ku
# coding: UTF-8

#=mail2weblog のエントリーポイント
#
# 最初の著者:: トゥイー
# リポジトリ情報:: $Id: m2w.rb 305 2011-09-25 03:10:54Z toy_dev $
# 著作権:: Copyright (C) Ownway.info, 2011. All rights reserved.
# ライセンス:: CPL(Common Public Licence)

# mail2weblog のルートパスを設定する
M2W_ROOT_PATH = File.dirname(File.expand_path(__FILE__))

# LOAD_PATH に必要なフォルダを追加する
$:.unshift(M2W_ROOT_PATH + "/")
$:.unshift(M2W_ROOT_PATH + "/lib")

require 'm2w_configurator'
require 'm2w_mail_parser'
require 'm2w_mail_sender'
require 'm2w_plugin_manager'
require 'weblog_command'
require 'mui/mail_command_processor'

start_time = Time.now.to_i

begin
	mail_parser = Mail2WeblogMailParser.new(
		M2W_SUBJECT_HEADER,
		M2W_SUBJECT_SEPARATOR_REGEX)
	mail_sender = Mail2WeblogMailSender.new(
		M2W_SMTP_SERVER_ADDRESS,
		M2W_SMTP_SERVER_PORT,
		M2W_SMTP_SECURITY_TYPE,
		M2W_SMTP_SERVER_USER,
		M2W_SMTP_SERVER_PASSWORD,
		M2W_POP3_BEFORE_SMTP_SERVER_ADDRESS,
		M2W_POP3_BEFORE_SMTP_SERVER_PORT,
		M2W_POP3_BEFORE_SMTP_SERVER_USER,
		M2W_POP3_BEFORE_SMTP_SERVER_PASSWORD,
		M2W_SUBJECT_HEADER,
		M2W_SUBJECT_SEPARATOR,
		MailProcessor.parse_mail_address(M2W_REPLY_ADDRESS))
	mail_command_processor = MailCommandProcessor.new(
		M2W_POP3_SERVER_ADDRESS,
		M2W_POP3_SERVER_PORT,
		M2W_POP3_SERVER_USER,
		M2W_POP3_SERVER_PASSWORD)
	mail_command = WeblogCommand.new(
		M2W_WEBLOG_EXTENSION,
		mail_parser,
		mail_sender,
		Mail2WeblogPluginManager.new(M2W_COMMAND_PLUGIN_CONF, M2W_FORMAT_PLUGIN_CONF, M2W_WEBLOG_PLUGIN_CONF),
		start_time,
		M2W_SYSTEM_PROCESSING_TIME)
	mail_command_processor.add(mail_command)

	if M2W_SYSTEM_LOCKFILE != nil then
		lockfile = File.open(M2W_ROOT_PATH + "/" + M2W_SYSTEM_LOCKFILE, 'w')
		lockfile.flock(File::LOCK_EX)
	end

	while true
		mail_command_processor.run

		if !(M2W_SYSTEM_CONTINOUS_PROCESSING && M2W_SYSTEM_PROCESSING_TIME >= 0 && M2W_SYSTEM_PROCESSING_TIME >= Time.now.to_i - start_time) then
			break
		end

		if M2W_SYSTEM_CONTINOUS_PROCESSING && M2W_SYSTEM_PROCESSING_TIME >= 0 && M2W_SYSTEM_MAIL_RECEIVER_SLEEPING_TIME >= 0 then
			sleep(M2W_SYSTEM_MAIL_RECEIVER_SLEEPING_TIME)
		end
	end
rescue Mail2WeblogTimeout => tout
	M2W_LOGGER.INFO(tout.message)
rescue => e
	M2W_LOGGER.error(e)
	exit(1)
end

puts "Content-type: text/plain\n\n"
puts "このプログラムは、cron などのスケジュール管理ソフトから呼び出すのが妥当です。"
