# coding: UTF-8

require 'plugin/weblog/lib/standard_command'
require 'plugin/weblog/lib/weblog_plugin_error'
require 'plugin/weblog/meta_weblog/lib/meta_weblog_post_common'

#=metaWeblog API を使った標準のブログ編集投稿用ウェブログプラグイン
#
# 最初の著者:: トゥイー
# リポジトリ情報:: $Id: meta_weblog_edit.rb 294 2011-09-24 01:13:10Z toy_dev $
# 著作権:: Copyright (C) Ownway.info, 2011. All rights reserved.
# ライセンス:: CPL(Common Public Licence)
class MetaWeblogEdit_WeblogPlugin < StandardCommand_WeblogPlugin
	include MetaWeblogPostCommon

	def header_alias
		return M2W_META_WEBLOG_POST_HEADER_ALIAS
	end

	def edit(original_content, header, content, attachments, blogid)
		M2W_LOGGER.debug("Start  #{self.class.name}#edit ... endpoint = #{@parent.endpoint}, blogid = #{blogid}, header = #{header.to_s}")

		client = @parent.client
		formatter = @parent.formatter

		postid = header['postid']
		header.delete('postid')

		if postid != nil then
			before(client, header, content, attachments, blogid, postid)

			vars = {:blogid => blogid, :postid => postid, :encoding => @parent.encoding, M2W_FORMAT_PLUGIN_CONF_HEADER_NAME => header[M2W_FORMAT_PLUGIN_CONF_HEADER_NAME]}

			# 事前フォーマットする
			begin
				(preformat_contents, preformat_errors, preformat_warnings) = formatter.preformat(
					content,
					original_content,
					vars,
					attachments != nil && !attachments.empty?)
				header['description'] = preformat_contents
				@errors.concat(preformat_errors)
				@warnings.concat(preformat_warnings)
			rescue => e
				M2W_LOGGER.error("ブログ記事の事前書式変換に失敗しました。")
				M2W_LOGGER.error(e)
				@errors.push(WeblogPluginError.new(
					"事前書式変換エラー(メッセージ = #{e.message})",
					"ブログ記事の事前書式変換に失敗しました(メッセージ = #{e.message})。"))
				return false
			end

			# 投稿日付を取得する
			if header['dateCreated'] != nil then
				dateCreated = @parent.parse_date(header['dateCreated'])
				header['dateCreated'] = @parent.get_date(dateCreated)
			else
				dateCreated = Time.now
			end

			# 添付ファイルを投稿する
			(attachment_urls, thumbnail_urls) = post_attachments(client, @parent.username, @parent.password, blogid, postid, header, vars, attachments, dateCreated, @errors, @warnings)
			@report[:attachment_urls] = attachment_urls
			@report[:thumbnail_urls] = thumbnail_urls

			# 事後フォーマットする
			if formatter.has_plugin_content || attachment_urls != nil then
				vars = vars.merge(header)
				vars[:attachment_urls] = attachment_urls
				vars[:thumbnail_urls] = thumbnail_urls

				begin
					(postformat_contents, postformat_errors, postformat_warinings) = formatter.postformat(header['description'], vars)
				rescue => e
					M2W_LOGGER.error("ブログ記事の事前書式変換に失敗しました。")
					M2W_LOGGER.error(e)
					@errors.push(WeblogPluginError.new(
						"事後書式変換エラー(メッセージ = #{e.message})",
						"ブログ記事の事前書式変換に失敗しました(メッセージ = #{e.message})。"))
					return false
				end
				header['description'] = postformat_contents
				@errors.concat(postformat_errors)
				@warnings.concat(postformat_warinings)
			end

			# ブログ記事を編集投稿する
			begin
				M2W_LOGGER.debug("Start  metaWeblog.editPost ...")
				edit_result = client.call("metaWeblog.editPost", postid, @parent.username, @parent.password, header, 1)
				@report[:postid] = postid
				@report[:edit_result] = edit_result
				M2W_LOGGER.debug("Finish metaWeblog.editPost ... result = #{edit_result}")
			rescue => e
				M2W_LOGGER.error("ブログ記事の書式変換後の投稿に失敗しました。")
				M2W_LOGGER.error(e)
				@errors.push(WeblogPluginError.new(
					"ブログ記事編集エラー(メッセージ = #{e.message})",
					"ブログ記事の編集に失敗しました(メッセージ = #{e.message})。"))
				return false
			end

			after(client, header, content, attachments, blogid, postid)

			@report[:title] = header['title']

			M2W_LOGGER.debug("Finish #{self.class.name}#edit")
			return true
		else
			M2W_LOGGER.debug("Finish #{self.class.name}#edit")
			return false
		end
	end

	def before(client, header, content, attachments, blogid, postid)
	end

	def after(client, header, content, attachments, blogid, postid)
	end

end
