# coding: UTF-8

require 'plugin/weblog/lib/standard_command'

require 'open-uri'

#=metaWeblog API を使った標準のブログ記事取得用ウェブログプラグイン
#
# 最初の著者:: トゥイー
# リポジトリ情報:: $Id: meta_weblog_get.rb 274 2011-09-19 07:32:08Z toy_dev $
# 著作権:: Copyright (C) Ownway.info, 2011. All rights reserved.
# ライセンス:: CPL(Common Public Licence)
class MetaWeblogGet_WeblogPlugin < StandardCommand_WeblogPlugin

	def header_alias
		return {}
	end

	def get(header, content, attachments, param)
		M2W_LOGGER.debug("Start  #{self.class.name}#get ... endpoint = #{@parent.endpoint}, header = #{header.to_s}")
		result = false

		client = @parent.client
		formatter = @parent.formatter

		postid = header['postid']

		before(client, header, content, attachments, param, postid)

		# ブログ記事情報を取得する
		begin
			M2W_LOGGER.debug("Start  metaWeblog.getPost ... postid = #{postid}")
			post = client.call("metaWeblog.getPost", postid, @parent.username, @parent.password)
			@report[:link] = post['link']
			M2W_LOGGER.debug("Finish metaWeblog.getPost")
		rescue => e
			M2W_LOGGER.error("ブログ記事の取得に失敗しました。")
			M2W_LOGGER.error(e)
			@errors.push(WeblogPluginError.new(
				"ブログ記事取得エラー(メッセージ = #{e.message})",
				"ブログ記事の取得に失敗しました(メッセージ = #{e.message})。"))
			return false
		end

		# ブログ記事情報を解析する
		(version, wikicontent) = formatter.get_uncommentout_content(post['description'])
		if wikicontent != nil then
			@report[:wikicontent] = wikicontent
			result = true
		end
		attachment_urls = formatter.get_uncommentout_attachment_urls_content(post['description'])
		@report[:attachments] = get_attachments(attachment_urls)

		after(client, header, content, attachments, param, postid)

		M2W_LOGGER.debug("Finish #{self.class.name}#get ... result = #{result}")
		return result
	end

	def before(client, header, content, attachments, param, postid)
	end

	def after(client, header, content, attachments, param, postid)
	end

	def get_attachments(urls)
		M2W_LOGGER.debug("Start  #{self.class.name}#get_attachments ... urls = #{urls.to_s}")
		result = []

		urls.each do |index, url|
			begin
				open(url, 'rb') do |f|
					attachment_content = f.gets(nil)
					if %r!/([^/]+)$! =~ url then
						attachment_filename = $1
						M2W_LOGGER.debug("attachment_filename ... #{attachment_filename}")
						result.push(MailAttachment.new(attachment_filename, attachment_content))
					end
				end
			rescue => e
				M2W_LOGGER.error("添付ファイルの取得に失敗しました。")
				M2W_LOGGER.error(e)
				@errors.push(WeblogPluginError.new(
					"添付ファイル取得エラー(メッセージ = #{e.message})",
					"添付ファイルの取得に失敗しました(メッセージ = #{e.message})。"))
			end
		end

		M2W_LOGGER.debug("Finish #{self.class.name}#get_attachments ... result = #{result.size}")
		return result
	end

end
